from flask import Flask, request, jsonify, render_template
from flask_socketio import SocketIO, emit
from data_fetcher import fetch_historical_data, fetch_realtime_data
from strategy import generate_signals
from backtester import execute_backtest, analyze_performance
from notifier import send_email_custom
import pandas as pd
import yaml
import logging
import time
import random
import os

# Get the directory of the current script
script_dir = os.path.dirname(__file__)
config_path = os.path.join(script_dir, 'config.yaml')

# Load configuration from YAML file
with open(config_path, 'r', encoding='utf-8') as file:
    config = yaml.safe_load(file)

# 로그 설정
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

# config.yaml에서 설정 로드
config_file_path = 'config.yaml'
with open(config_file_path, 'r', encoding='utf-8') as file:
    config = yaml.safe_load(file)

email_config = config['email_config']

app = Flask(__name__)
socketio = SocketIO(app)

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/dashboard')
def dashboard():
    return render_template('dashboard.html')

@app.route('/get_balance', methods=['POST'])
def get_balance():
    data = request.json
    cano = data.get('cano')
    acnt_prdt_cd = data.get('acnt_prdt_cd')
    balance = {"cano": cano, "acnt_prdt_cd": acnt_prdt_cd, "balance": 1000000}
    return jsonify(balance)

@app.route('/run_backtest', methods=['POST'])
def run_backtest():
    data = request.json.get('data')
    df = pd.read_json(data)
    signals = generate_signals(df)
    portfolio = execute_backtest(signals)
    performance_metrics = analyze_performance(portfolio)
    return jsonify(performance_metrics)

@app.route('/start_trader', methods=['POST'])
def start_trader():
    return jsonify({"status": "Real-time trader started"})

@app.route('/send_email', methods=['POST'])
def send_email():
    data = request.json
    subject = data.get('subject')
    body = data.get('body')
    to_email = data.get('to_email')
    send_email_custom(
        subject=subject,
        body=body,
        to_email=to_email,
        from_email=email_config['from_email'],
        smtp_server=email_config['smtp_server'],
        smtp_port=email_config['smtp_port'],
        smtp_user=email_config['smtp_user'],
        smtp_password=email_config['smtp_password']
    )
    return jsonify({"status": "Email sent successfully"})

@socketio.on('connect')
def handle_connect():
    print('Client connected')

@socketio.on('disconnect')
def handle_disconnect():
    print('Client disconnected')

def send_realtime_data():
    while True:
        data = {
            'time': time.strftime('%H:%M:%S'),
            'value': random.uniform(100, 200)
        }
        socketio.emit('realtime_data', data)
        time.sleep(1)

if __name__ == '__main__':
    socketio.start_background_task(send_realtime_data)
    socketio.run(app, debug=True)
