import pandas as pd
import numpy as np

def execute_backtest(signals, initial_balance=10000, commission=0.001, slippage=0.0005, max_position_size=0.1, max_loss_limit=0.1, trailing_stop_loss=0.02):
    positions = pd.DataFrame(index=signals.index).fillna(0.0)
    positions['price'] = signals['price']
    positions['position'] = signals['signal']

    portfolio = pd.DataFrame(index=signals.index)
    portfolio['holdings'] = positions['position'] * positions['price']
    portfolio['cash'] = initial_balance - (positions['position'].diff() * positions['price'] * (1 + commission + slippage)).cumsum()
    portfolio['total'] = portfolio['cash'] + portfolio['holdings']

    # 최대 포지션 크기 제한
    portfolio['position_size'] = positions['position'] * positions['price'] / portfolio['total']
    portfolio['position_size'] = np.minimum(portfolio['position_size'], max_position_size)

    # 최대 손실 한도 및 트레일링 스탑 로스
    drawdown = (portfolio['total'] / portfolio['total'].cummax()) - 1
    if drawdown.min() < -max_loss_limit:
        portfolio.loc[portfolio.index > drawdown.idxmin(), 'holdings'] = 0
        portfolio.loc[portfolio.index > drawdown.idxmin(), 'cash'] = portfolio['total'][drawdown.idxmin()]

    # 트레일링 스탑 로스
    trailing_stop = portfolio['total'].cummax() * (1 - trailing_stop_loss)
    portfolio['stop_loss_trigger'] = portfolio['total'] < trailing_stop
    portfolio.loc[portfolio['stop_loss_trigger'], 'holdings'] = 0
    portfolio.loc[portfolio['stop_loss_trigger'], 'cash'] = portfolio['total'][portfolio['stop_loss_trigger']]

    return portfolio

# 최대 낙폭 계산
def calculate_max_drawdown(series):
    peak = series.expanding(min_periods=1).max()
    drawdown = (series / peak) - 1
    max_drawdown = drawdown.min()
    return max_drawdown

# 성과 분석
def analyze_performance(portfolio):
    returns = portfolio['total'].pct_change()
    total_return = portfolio['total'].iloc[-1] / portfolio['total'].iloc[0] - 1
    annual_return = returns.mean() * 252
    annual_volatility = returns.std() * np.sqrt(252)
    sharpe_ratio = annual_return / annual_volatility
    mdd = calculate_max_drawdown(portfolio['total'])
    calmar_ratio = annual_return / abs(mdd)
    sortino_ratio = annual_return / returns[returns < 0].std() * np.sqrt(252)
    omega_ratio = returns[returns > 0].sum() / abs(returns[returns < 0].sum())

    performance_metrics = {
        'Total Return': total_return,
        'Annual Return': annual_return,
        'Annual Volatility': annual_volatility,
        'Sharpe Ratio': sharpe_ratio,
        'Max Drawdown': mdd,
        'Calmar Ratio': calmar_ratio,
        'Sortino Ratio': sortino_ratio,
        'Omega Ratio': omega_ratio
    }

    for metric, value in performance_metrics.items():
        print(f"{metric}: {value:.2f}")

    return performance_metrics
