import requests
import pandas as pd
import yaml
import logging

# 로그 설정
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

# config.yaml에서 API 설정 로드
config_file_path = 'config.yaml'
with open(config_file_path, 'r', encoding='utf-8') as file:
    config = yaml.safe_load(file)

api_config = config['api_config']

def get_api_url(endpoint):
    return f"{api_config['base_url']}{endpoint}"

def get_headers(access_token):
    return {
        'content-type': 'application/json; charset=utf-8',
        'authorization': f'Bearer {access_token}',
        'appkey': api_config['appkey'],
        'appsecret': api_config['appsecret'],
        'tr_id': 'FHKST01010400'
    }

def get_access_token():
    url = get_api_url(api_config['access_token']['url'])
    headers = api_config['access_token']['headers']
    body = api_config['access_token']['body']
    
    response = requests.post(url, headers=headers, json=body)
    if response.status_code == 200:
        logging.info("Access token obtained successfully.")
        return response.json()['access_token']
    else:
        error_message = response.json().get('msg', 'Unknown error occurred.')
        raise Exception(f"Failed to get access token: {error_message}")

def fetch_historical_data(symbol, market_code, start_date, end_date):
    access_token = get_access_token()
    
    url = get_api_url('/uapi/domestic-stock/v1/quotations/inquire-daily-price')
    headers = get_headers(access_token)
    params = {
        'FID_COND_MRKT_DIV_CODE': market_code,  # KOSPI: J, KOSDAQ: K
        'FID_INPUT_ISCD': symbol,
        'FID_PERIOD_DIV_CODE': 'D',  # D: 일간
        'FID_ORG_ADJ_PRC': '1'  # 1: 수정주가
    }

    response = requests.get(url, headers=headers, params=params)
    if response.status_code == 200:
        data = response.json()['output']
        df = pd.DataFrame(data)
        df['stck_bsop_date'] = pd.to_datetime(df['stck_bsop_date'])
        df.set_index('stck_bsop_date', inplace=True)
        df = df[(df.index >= start_date) & (df.index <= end_date)]
        df = df.sort_index()
        
        # 데이터 타입 변환 및 결측치 처리
        df = df.apply(pd.to_numeric, errors='coerce').dropna()
        
        logging.info(f"Fetched historical data for {symbol} from {start_date} to {end_date}.")
        return df
    else:
        error_message = response.json().get('msg', 'Unknown error occurred.')
        raise Exception(f"Failed to fetch historical data: {error_message}")

def fetch_realtime_data(symbol, market_code):
    access_token = get_access_token()
    
    url = get_api_url('/uapi/domestic-stock/v1/quotations/inquire-price')
    headers = get_headers(access_token)
    params = {
        'FID_COND_MRKT_DIV_CODE': market_code,
        'FID_INPUT_ISCD': symbol
    }

    response = requests.get(url, headers=headers, params=params)
    if response.status_code == 200:
        data = response.json()['output']
        df = pd.DataFrame([data])
        df['stck_bsop_date'] = pd.to_datetime(df['stck_bsop_date'])
        df.set_index('stck_bsop_date', inplace=True)
        df = df.apply(pd.to_numeric, errors='coerce').dropna()
        
        logging.info(f"Fetched real-time data for {symbol}.")
        return df
    else:
        error_message = response.json().get('msg', 'Unknown error occurred.')
        raise Exception(f"Failed to fetch real-time data: {error_message}")
